import InputMethodKit
from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure


class TestIMKServer(TestCase):
    @expectedFailure
    def testBrokenConstants(self):
        # The definitions below are defined on 10.5, but not actually
        # exported by the framework.
        #
        # See also: Radar #6783035
        self.assertIsInstance(InputMethodKit.IMKModeDictionary, str)
        self.assertIsInstance(InputMethodKit.IMKDelegateClass, str)
        self.assertIsInstance(InputMethodKit.IMKControllerClass, str)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(InputMethodKit.IMKServer.paletteWillTerminate)
        self.assertResultIsBOOL(InputMethodKit.IMKServer.lastKeyEventWasDeadKey)



"""
Some simple tests to check that the framework is properly wrapped.
"""
import InputMethodKit
from PyObjCTools.TestSupport import TestCase
import objc


class TestInputMethodKit(TestCase):
    def testClasses(self):

        self.assertHasAttr(InputMethodKit, "IMKInputController")
        self.assertIsInstance(InputMethodKit.IMKInputController, objc.objc_class)

        # 10.5
        self.assertHasAttr(InputMethodKit, "IMKCandidates")
        self.assertIsInstance(InputMethodKit.IMKCandidates, objc.objc_class)

    def testValues(self):
        self.assertHasAttr(InputMethodKit, "kIMKScrollingGridCandidatePanel")
        self.assertIsInstance(InputMethodKit.kIMKScrollingGridCandidatePanel, int)
        self.assertEqual(InputMethodKit.kIMKScrollingGridCandidatePanel, 2)

    def testVariables(self):
        self.assertHasAttr(InputMethodKit, "IMKCandidatesOpacityAttributeName")
        self.assertIsInstance(InputMethodKit.IMKCandidatesOpacityAttributeName, str)

    def testProtocols2(self):
        objc.protocolNamed("IMKMouseHandling")
        objc.protocolNamed("IMKStateSetting")


class TestCallableMetadata(TestCase):
    def test_callable_metadata_is_sane(self):
        self.assertCallableMetadataIsSane(InputMethodKit)



# This file is generated by objective.metadata
#
# Last update: Sun Feb 20 18:54:52 2022
#
# flake8: noqa

import objc, sys
from typing import NewType

if sys.maxsize > 2**32:

    def sel32or64(a, b):
        return b

else:

    def sel32or64(a, b):
        return a


if objc.arch == "arm64":

    def selAorI(a, b):
        return a

else:

    def selAorI(a, b):
        return b


misc = {}
constants = """$IMKCandidatesOpacityAttributeName$IMKCandidatesSendServerKeyEventFirst$IMKControllerClass$IMKDelegateClass$IMKModeDictionary$kIMKCommandClientName$kIMKCommandMenuItemName$"""
enums = """$kIMKAnnotation@1$kIMKLocateCandidatesAboveHint@1$kIMKLocateCandidatesBelowHint@2$kIMKLocateCandidatesLeftHint@3$kIMKLocateCandidatesRightHint@4$kIMKMain@0$kIMKScrollingGridCandidatePanel@2$kIMKSingleColumnScrollingCandidatePanel@1$kIMKSingleRowSteppingCandidatePanel@3$kIMKSubList@2$"""
misc.update({})
misc.update({})
r = objc.registerMetaDataForSelector
objc._updatingMetadata(True)
try:
    r(b"IMKCandidates", b"dismissesAutomatically", {"retval": {"type": "Z"}})
    r(b"IMKCandidates", b"isVisible", {"retval": {"type": "Z"}})
    r(b"IMKCandidates", b"selectCandidateWithIdentifier:", {"retval": {"type": b"Z"}})
    r(
        b"IMKCandidates",
        b"selectionKeysKeylayout",
        {"retval": {"type": "^{__TISInputSource=}"}},
    )
    r(
        b"IMKCandidates",
        b"setDismissesAutomatically:",
        {"arguments": {2: {"type": "Z"}}},
    )
    r(
        b"IMKCandidates",
        b"setSelectionKeysKeylayout:",
        {"arguments": {2: {"type": "^{__TISInputSource=}"}}},
    )
    r(
        b"IMKInputController",
        b"doCommandBySelector:commandDictionary:",
        {"arguments": {2: {"type": ":", "sel_of_type": b"v@:@"}}},
    )
    r(b"IMKServer", b"lastKeyEventWasDeadKey", {"retval": {"type": b"Z"}})
    r(b"IMKServer", b"paletteWillTerminate", {"retval": {"type": b"Z"}})
    r(
        b"NSObject",
        b"activateServer:",
        {"required": True, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"candidates:",
        {"retval": {"type": b"@"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"commitComposition:",
        {"retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"composedString:",
        {"retval": {"type": b"@"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"deactivateServer:",
        {"required": True, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"didCommandBySelector:client:",
        {"retval": {"type": "Z"}, "arguments": {2: {"type": ":"}, 3: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"handleEvent:client:",
        {"retval": {"type": "Z"}, "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"inputText:client:",
        {"retval": {"type": "Z"}, "arguments": {2: {"type": b"@"}, 3: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"inputText:key:modifiers:client:",
        {
            "retval": {"type": "Z"},
            "arguments": {
                2: {"type": b"@"},
                3: {"type": b"q"},
                4: {"type": b"Q"},
                5: {"type": b"@"},
            },
        },
    )
    r(
        b"NSObject",
        b"modes:",
        {"required": True, "retval": {"type": b"@"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"mouseDownOnCharacterIndex:coordinate:withModifier:continueTracking:client:",
        {
            "required": True,
            "retval": {"type": "Z"},
            "arguments": {
                2: {"type": sel32or64(b"I", b"Q")},
                3: {"type": sel32or64(b"{_NSPoint=ff}", b"{CGPoint=dd}")},
                4: {"type": sel32or64(b"I", b"Q")},
                5: {"type": "^Z", "type_modifier": b"o"},
                6: {"type": b"@"},
            },
        },
    )
    r(
        b"NSObject",
        b"mouseMovedOnCharacterIndex:coordinate:withModifier:client:",
        {
            "required": True,
            "retval": {"type": "Z"},
            "arguments": {
                2: {"type": sel32or64(b"I", b"Q")},
                3: {"type": sel32or64(b"{_NSPoint=ff}", b"{CGPoint=dd}")},
                4: {"type": sel32or64(b"I", b"Q")},
                5: {"type": b"@"},
            },
        },
    )
    r(
        b"NSObject",
        b"mouseUpOnCharacterIndex:coordinate:withModifier:client:",
        {
            "required": True,
            "retval": {"type": "Z"},
            "arguments": {
                2: {"type": sel32or64(b"I", b"Q")},
                3: {"type": sel32or64(b"{_NSPoint=ff}", b"{CGPoint=dd}")},
                4: {"type": sel32or64(b"I", b"Q")},
                5: {"type": b"@"},
            },
        },
    )
    r(
        b"NSObject",
        b"originalString:",
        {"retval": {"type": b"@"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"recognizedEvents:",
        {"required": True, "retval": {"type": b"Q"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"setValue:forTag:client:",
        {
            "required": True,
            "retval": {"type": b"v"},
            "arguments": {2: {"type": b"@"}, 3: {"type": b"q"}, 4: {"type": b"@"}},
        },
    )
    r(
        b"NSObject",
        b"showPreferences:",
        {"required": True, "retval": {"type": b"v"}, "arguments": {2: {"type": b"@"}}},
    )
    r(
        b"NSObject",
        b"valueForTag:client:",
        {
            "required": True,
            "retval": {"type": b"@"},
            "arguments": {2: {"type": b"q"}, 3: {"type": b"@"}},
        },
    )
finally:
    objc._updatingMetadata(False)
protocols = {
    "IMKServerInput": objc.informal_protocol(
        "IMKServerInput",
        [
            objc.selector(None, b"inputText:client:", b"Z@:@@", isRequired=False),
            objc.selector(None, b"candidates:", b"@@:@", isRequired=False),
            objc.selector(
                None, b"didCommandBySelector:client:", b"Z@::@", isRequired=False
            ),
            objc.selector(None, b"handleEvent:client:", b"Z@:@@", isRequired=False),
            objc.selector(None, b"composedString:", b"@@:@", isRequired=False),
            objc.selector(
                None, b"inputText:key:modifiers:client:", b"Z@:@qQ@", isRequired=False
            ),
            objc.selector(None, b"commitComposition:", b"v@:@", isRequired=False),
            objc.selector(None, b"originalString:", b"@@:@", isRequired=False),
        ],
    )
}
expressions = {}

# END OF FILE



"""
Python mapping for the InputMethodKit framework.

This module does not contain docstrings for the wrapped code, check Apple's
documentation for details on how to use these functions and classes.
"""
import sys

import Foundation
import objc
from InputMethodKit import _metadata
from InputMethodKit import _InputMethodKit

sys.modules["InputMethodKit"] = mod = objc.ObjCLazyModule(
    "InputMethodKit",
    "com.apple.InputMethodKit",
    objc.pathForFramework("/System/Library/Frameworks/InputMethodKit.framework"),
    _metadata.__dict__,
    None,
    {
        "__doc__": __doc__,
        "__path__": __path__,
        "__loader__": globals().get("__loader__", None),
        "objc": objc,
    },
    (_InputMethodKit, Foundation),
)


del sys.modules["InputMethodKit._metadata"]



import InputMethodKit
from PyObjCTools.TestSupport import TestCase
import objc


class TestIMKInputControllerHelper(InputMethodKit.NSObject):
    def inputText_key_modifiers_client_(self, s, k, m, snd):
        return True

    def inputText_client_(self, s, snd):
        return True

    def handleEvent_client_(self, s, snd):
        return True

    def didCommandBySelector_client_(self, s, snd):
        return False

    # IMKMouseHandling

    def mouseDownOnCharacterIndex_coordinate_withModifier_continueTracking_client_(
        self, v1, v2, v3, v4, v5
    ):
        return True

    def mouseUpOnCharacterIndex_coordinate_withModifier_client_(self, v1, v2, v3, v4):
        return True

    def mouseMovedOnCharacterIndex_coordinate_withModifier_client_(
        self, v1, v2, v3, v4
    ):
        return True


class TestIMKInputController(TestCase):
    def testConstants(self):
        self.assertIsInstance(InputMethodKit.kIMKCommandMenuItemName, str)
        self.assertIsInstance(InputMethodKit.kIMKCommandClientName, str)

    def testProtocols(self):
        # self.assertIsInstance(protocols.IMKServerInput, objc.informal_protocol)

        self.assertResultIsBOOL(
            TestIMKInputControllerHelper.inputText_key_modifiers_client_
        )
        self.assertArgHasType(
            TestIMKInputControllerHelper.inputText_key_modifiers_client_,
            1,
            objc._C_NSInteger,
        )

        self.assertResultIsBOOL(TestIMKInputControllerHelper.inputText_client_)
        self.assertResultIsBOOL(TestIMKInputControllerHelper.handleEvent_client_)
        self.assertResultIsBOOL(
            TestIMKInputControllerHelper.didCommandBySelector_client_
        )
        self.assertArgHasType(
            TestIMKInputControllerHelper.didCommandBySelector_client_, 0, objc._C_SEL
        )

    def testIMKMouseHandling(self):
        self.assertResultIsBOOL(
            TestIMKInputControllerHelper.mouseDownOnCharacterIndex_coordinate_withModifier_continueTracking_client_
        )
        self.assertArgHasType(
            TestIMKInputControllerHelper.mouseDownOnCharacterIndex_coordinate_withModifier_continueTracking_client_,
            0,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestIMKInputControllerHelper.mouseDownOnCharacterIndex_coordinate_withModifier_continueTracking_client_,
            1,
            InputMethodKit.NSPoint.__typestr__,
        )

        self.assertArgHasType(
            TestIMKInputControllerHelper.mouseDownOnCharacterIndex_coordinate_withModifier_continueTracking_client_,
            2,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestIMKInputControllerHelper.mouseDownOnCharacterIndex_coordinate_withModifier_continueTracking_client_,
            3,
            objc._C_OUT + objc._C_PTR + objc._C_NSBOOL,
        )

        self.assertResultIsBOOL(
            TestIMKInputControllerHelper.mouseUpOnCharacterIndex_coordinate_withModifier_client_
        )
        self.assertArgHasType(
            TestIMKInputControllerHelper.mouseUpOnCharacterIndex_coordinate_withModifier_client_,
            0,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestIMKInputControllerHelper.mouseUpOnCharacterIndex_coordinate_withModifier_client_,
            1,
            InputMethodKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestIMKInputControllerHelper.mouseUpOnCharacterIndex_coordinate_withModifier_client_,
            2,
            objc._C_NSUInteger,
        )

        self.assertResultIsBOOL(
            TestIMKInputControllerHelper.mouseMovedOnCharacterIndex_coordinate_withModifier_client_
        )
        self.assertArgHasType(
            TestIMKInputControllerHelper.mouseMovedOnCharacterIndex_coordinate_withModifier_client_,
            0,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestIMKInputControllerHelper.mouseMovedOnCharacterIndex_coordinate_withModifier_client_,
            1,
            InputMethodKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestIMKInputControllerHelper.mouseMovedOnCharacterIndex_coordinate_withModifier_client_,
            2,
            objc._C_NSUInteger,
        )

    def testMethods(self):
        self.assertArgIsSEL(
            InputMethodKit.IMKInputController.doCommandBySelector_commandDictionary_,
            0,
            b"v@:@",
        )



import InputMethodKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestIMKCandidates(TestCase):
    def testConstants(self):
        self.assertEqual(InputMethodKit.kIMKSingleColumnScrollingCandidatePanel, 1)
        self.assertEqual(InputMethodKit.kIMKScrollingGridCandidatePanel, 2)
        self.assertEqual(InputMethodKit.kIMKSingleRowSteppingCandidatePanel, 3)

        self.assertEqual(InputMethodKit.kIMKLocateCandidatesAboveHint, 1)
        self.assertEqual(InputMethodKit.kIMKLocateCandidatesBelowHint, 2)
        self.assertEqual(InputMethodKit.kIMKLocateCandidatesLeftHint, 3)
        self.assertEqual(InputMethodKit.kIMKLocateCandidatesRightHint, 4)

        self.assertEqual(InputMethodKit.kIMKMain, 0)
        self.assertEqual(InputMethodKit.kIMKAnnotation, 1)
        self.assertEqual(InputMethodKit.kIMKSubList, 2)

        self.assertIsInstance(InputMethodKit.IMKCandidatesOpacityAttributeName, str)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertIsInstance(InputMethodKit.IMKCandidatesSendServerKeyEventFirst, str)

    def testMethods(self):
        self.assertResultIsBOOL(InputMethodKit.IMKCandidates.isVisible)
        self.assertResultIsBOOL(InputMethodKit.IMKCandidates.dismissesAutomatically)
        self.assertArgIsBOOL(InputMethodKit.IMKCandidates.setDismissesAutomatically_, 0)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(
            InputMethodKit.IMKCandidates.selectCandidateWithIdentifier_
        )
